@@ -735,7 +735,7 @@ class TestListArray : public ::testing::Test {
         ArrayFromJSON(type, "[[1, 2], [3], [4], null, [5], [], [6]]"));
     auto sliced_list_array =
         std::dynamic_pointer_cast<ArrayType>(list_array->Slice(3, 4));
-    ASSERT_OK_AND_ASSIGN(auto flattened, list_array->Flatten());
+    ASSERT_OK_AND_ASSIGN(auto flattened, sliced_list_array->Flatten());
     ASSERT_OK(flattened->ValidateFull());
     // Note the difference between values() and Flatten().
     EXPECT_TRUE(flattened->Equals(ArrayFromJSON(int32(), "[5, 6]")));
@@ -763,6 +763,52 @@ class TestListArray : public ::testing::Test {
         << flattened->ToString();
   }
 
+  void TestFlattenRecursively() {
+    auto inner_type = std::make_shared<T>(int32());
+    auto type = std::make_shared<T>(inner_type);
+
+    // List types with two nested level: list<list<int32>>
+    auto nested_list_array = std::dynamic_pointer_cast<ArrayType>(ArrayFromJSON(type, R"([
+      [[0, 1, 2], null, [3, null]],
+      [null],
+      [[2, 9], [4], [], [6, 5]]
+    ])"));
+    ASSERT_OK_AND_ASSIGN(auto flattened, nested_list_array->FlattenRecursively());
+    ASSERT_OK(flattened->ValidateFull());
+    ASSERT_EQ(10, flattened->length());
+    ASSERT_TRUE(
+        flattened->Equals(ArrayFromJSON(int32(), "[0, 1, 2, 3, null, 2, 9, 4, 6, 5]")));
+
+    // Empty nested list should flatten until non-list type is reached
+    nested_list_array =
+        std::dynamic_pointer_cast<ArrayType>(ArrayFromJSON(type, R"([null])"));
+    ASSERT_OK_AND_ASSIGN(flattened, nested_list_array->FlattenRecursively());
+    ASSERT_TRUE(flattened->type()->Equals(int32()));
+
+    // List types with three nested level: list<list<fixed_size_list<int32, 2>>>
+    type = std::make_shared<T>(std::make_shared<T>(fixed_size_list(int32(), 2)));
+    nested_list_array = std::dynamic_pointer_cast<ArrayType>(ArrayFromJSON(type, R"([
+      [
+        [[null, 0]],
+        [[3, 7], null]
+      ],
+      [
+        [[4, null], [5, 8]],
+        [[8, null]],
+        null
+      ],
+      [
+        null
+      ]
+    ])"));
+    ASSERT_OK_AND_ASSIGN(flattened, nested_list_array->FlattenRecursively());
+    ASSERT_OK(flattened->ValidateFull());
+    ASSERT_EQ(10, flattened->length());
+    ASSERT_EQ(3, flattened->null_count());
+    ASSERT_TRUE(flattened->Equals(
+        ArrayFromJSON(int32(), "[null, 0, 3, 7, 4, null, 5, 8, 8, null]")));
+  }
+
   Status ValidateOffsetsAndSizes(int64_t length, std::vector<offset_type> offsets,
                                  std::vector<offset_type> sizes,
                                  std::shared_ptr<Array> values, int64_t offset = 0) {
@@ -925,10 +971,12 @@ TYPED_TEST(TestListArray, BuilderPreserveFieldName) {
 TYPED_TEST(TestListArray, FlattenSimple) { this->TestFlattenSimple(); }
 TYPED_TEST(TestListArray, FlattenNulls) { this->TestFlattenNulls(); }
 TYPED_TEST(TestListArray, FlattenAllEmpty) { this->TestFlattenAllEmpty(); }
+TYPED_TEST(TestListArray, FlattenSliced) { this->TestFlattenSliced(); }
 TYPED_TEST(TestListArray, FlattenZeroLength) { this->TestFlattenZeroLength(); }
 TYPED_TEST(TestListArray, TestFlattenNonEmptyBackingNulls) {
   this->TestFlattenNonEmptyBackingNulls();
 }
+TYPED_TEST(TestListArray, FlattenRecursively) { this->TestFlattenRecursively(); }
 
 TYPED_TEST(TestListArray, ValidateDimensions) { this->TestValidateDimensions(); }
 
@@ -1138,7 +1186,8 @@ TEST_F(TestMapArray, BuildingStringToInt) {
   std::vector<int32_t> offsets = {0, 2, 2, 3, 3};
   auto expected_keys = ArrayFromJSON(utf8(), R"(["joe", "mark", "cap"])");
   auto expected_values = ArrayFromJSON(int32(), "[0, null, 8]");
-  ASSERT_OK_AND_ASSIGN(auto expected_null_bitmap, internal::BytesToBits({1, 0, 1, 1}));
+  ASSERT_OK_AND_ASSIGN(auto expected_null_bitmap,
+                       internal::BytesToBits(std::vector<uint8_t>({1, 0, 1, 1})));
   MapArray expected(type, 4, Buffer::Wrap(offsets), expected_keys, expected_values,
                     expected_null_bitmap, 1);
 
@@ -1239,7 +1288,7 @@ TEST_F(TestMapArray, ValidateErrorNullKey) {
 }
 
 TEST_F(TestMapArray, FromArrays) {
-  std::shared_ptr<Array> offsets1, offsets2, offsets3, offsets4, keys, items;
+  std::shared_ptr<Array> offsets1, offsets2, offsets3, offsets4, offsets5, keys, items;
 
   std::vector<bool> offsets_is_valid3 = {true, false, true, true};
   std::vector<bool> offsets_is_valid4 = {true, true, false, true};
@@ -1294,6 +1343,20 @@ TEST_F(TestMapArray, FromArrays) {
   // Zero-length offsets
   ASSERT_RAISES(Invalid, MapArray::FromArrays(offsets1->Slice(0, 0), keys, items, pool_));
 
+  // Offseted offsets
+  ASSERT_OK_AND_ASSIGN(auto map5,
+                       MapArray::FromArrays(offsets1->Slice(1), keys, items, pool_));
+  ASSERT_OK(map5->Validate());
+
+  AssertArraysEqual(*expected1.Slice(1), *map5);
+
+  std::vector<MapType::offset_type> offset5_values = {2, 2, 6};
+  ArrayFromVector<OffsetType, offset_type>(offset5_values, &offsets5);
+  ASSERT_OK_AND_ASSIGN(auto map6, MapArray::FromArrays(offsets5, keys, items, pool_));
+  ASSERT_OK(map6->Validate());
+
+  AssertArraysEqual(*map5, *map6);
+
   // Offsets not the right type
   ASSERT_RAISES(TypeError, MapArray::FromArrays(keys, offsets1, items, pool_));
 
@@ -1306,6 +1369,35 @@ TEST_F(TestMapArray, FromArrays) {
   ASSERT_EQ(keys_with_null->length(), tmp_items->length());
   ASSERT_RAISES(Invalid,
                 MapArray::FromArrays(offsets1, keys_with_null, tmp_items, pool_));
+
+  // With null_bitmap and null_count=1
+  auto null_bitmap_1 = ArrayFromJSON(boolean(), "[1, 0, 1]")->data()->buffers[1];
+  ASSERT_OK_AND_ASSIGN(auto map7,
+                       MapArray::FromArrays(offsets1, keys, items, pool_, null_bitmap_1));
+  ASSERT_OK(map7->Validate());
+  MapArray expected7(map_type, length, offsets1->data()->buffers[1], keys, items,
+                     null_bitmap_1, 1);
+  ASSERT_EQ(map7->null_count(), 1);
+  AssertArraysEqual(expected7, *map7);
+
+  // With null_bitmap and null_count=2
+  auto null_bitmap_2 = ArrayFromJSON(boolean(), "[0, 1, 0]")->data()->buffers[1];
+  ASSERT_OK_AND_ASSIGN(auto map8,
+                       MapArray::FromArrays(offsets1, keys, items, pool_, null_bitmap_2));
+  ASSERT_OK(map8->Validate());
+  MapArray expected8(map_type, length, offsets1->data()->buffers[1], keys, items,
+                     null_bitmap_2, 2);
+  ASSERT_EQ(map8->null_count(), 2);
+  AssertArraysEqual(expected8, *map8);
+
+  // Null bitmap and offset with null
+  ASSERT_RAISES(Invalid, MapArray::FromArrays(offsets3, keys, items, pool_,
+                                              offsets3->data()->buffers[0]));
+
+  // Null bitmap and offset with offset
+  ASSERT_RAISES(NotImplemented,
+                MapArray::FromArrays(offsets1->Slice(2), keys, items, pool_,
+                                     offsets3->data()->buffers[0]));
 }
 
 TEST_F(TestMapArray, FromArraysEquality) {
@@ -1714,4 +1806,23 @@ TEST_F(TestFixedSizeListArray, Flatten) {
   }
 }
 
+TEST_F(TestFixedSizeListArray, FlattenRecursively) {
+  // Nested fixed-size list-array: fixed_size_list(fixed_size_list(int32, 2), 2)
+  auto inner_type = fixed_size_list(value_type_, 2);
+  type_ = fixed_size_list(inner_type, 2);
+
+  auto values = std::dynamic_pointer_cast<FixedSizeListArray>(ArrayFromJSON(type_, R"([
+    [[0, 1], [null, 3]],
+    [[7, null], [2, 5]],
+    [null, null]
+  ])"));
+  ASSERT_OK(values->ValidateFull());
+  ASSERT_OK_AND_ASSIGN(auto flattened, values->FlattenRecursively());
+  ASSERT_OK(flattened->ValidateFull());
+  ASSERT_EQ(8, flattened->length());
+  ASSERT_EQ(2, flattened->null_count());
+  AssertArraysEqual(*flattened,
+                    *ArrayFromJSON(value_type_, "[0, 1, null, 3, 7, null, 2, 5]"));
+}
+
 }  // namespace arrow
