@@ -233,10 +233,8 @@ class CSharpTester(Tester):
                 server.kill()
                 out, err = server.communicate()
                 raise RuntimeError(
-                    '.NET Flight server did not start properly, '
-                    'stdout: \n{}\n\nstderr:\n{}\n'.format(
-                        output + out.decode(), err.decode()
-                    )
+                    ".NET Flight server did not start properly, "
+                    f"stdout: \n{output + out.decode()}\n\nstderr:\n{err.decode()}\n"
                 )
             port = int(output.split(':')[-1])
             yield port
